@@ -37,8 +37,11 @@ def subblock_indices(n, outer_square_length, inner_square_length):
 	i = outer_square_length
 	j = inner_square_length
 
-	assert n < i * i, "n not contained in matrix"
-	assert j * j == i, "Currently only squares are supported"
+	if n >= i * i:
+		raise ValueError("n not contained in matrix")
+
+	if j * j != i:
+		raise ValueError("Currently only squares are supported")
 
 	x, y = (n % i // j, n // i // j)
 
