@@ -2,19 +2,21 @@ import requests
 
 class RasaRestWebhook(object):
 
-	scheme = "http"
-
-	def __init__(self, sender, netloc="localhost:5005", timeout=60):
-		# type: (str, str, int) -> None
+	def __init__(self, sender, scheme="http", netloc="localhost:5005", timeout=60):
+		# type: (str, str, str, int) -> None
 
 		self.sender = sender
+		self.scheme = scheme
 		self.netloc = netloc
 		self.timeout = timeout
 
+	def get_endpoint(self, path):
+		return self.scheme + "://" + self.netloc + path
+
 	def health(self):
 		# type: () -> dict
 
-		url = self.scheme + "://" + self.netloc + "/webhooks/rest/"
+		url = self.get_endpoint("/webhooks/rest/")
 
 		r = requests.get(url, timeout=self.timeout)
 		r.raise_for_status()
@@ -23,7 +25,7 @@ class RasaRestWebhook(object):
 	def send_message(self, message):
 		# type: (str, ) -> dict
 
-		url = self.scheme + "://" + self.netloc + "/webhooks/rest/webhook"
+		url = self.get_endpoint("/webhooks/rest/webhook")
 
 		r = requests.post(url, json={
 			"sender": self.sender,
